Attribute VB_Name = "WIN32API"
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' John Robbins
' Microsoft Systems Journal - August, 1997
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FILE         :     WIN32API.bas
' DESCRIPTION  :
'  The subset of the basic WIN32 API that VBDebug needs.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                             General Types
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Type STARTUPINFO
     cb As Long
     lpReserved As String
     lpDesktop As String
     lpTitle As String
     dwX As Long
     dwY As Long
     dwXSize As Long
     dwYSize As Long
     dwXCountChars As Long
     dwYCountChars As Long
     dwFillAttribute As Long
     dwFlags As Long
     wShowWindow As Integer
     cbReserved2 As Integer
     lpReserved2 As Byte
     hStdInput As Long
     hStdOutput As Long
     hStdError As Long
End Type

Type PROCESS_INFORMATION
     hProcess As Long
     hThread As Long
     dwProcessID As Long
     dwThreadID As Long
End Type

Type CRITICAL_SECTION
    LockCount As Long
    RecursionCount As Long
    OwningThread As Long
    LockSemaphore As Long
    Reserved As Long
End Type

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                         Constants and defines
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Public Const INFINITE As Long = &HFFFFFFFF

Public Const WAIT_OBJECT_0 As Long = &H0
Public Const WAIT_ABANDONED_0 As Long = &H80
Public Const WAIT_TIMEOUT As Long = &H102

Public Const STILL_ACTIVE As Long = &H103

Public Const WM_MBUTTONDOWN As Long = &H207

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                       Generally Useful Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Declare Function CloseHandle Lib "kernel32" _
         (ByVal hObject As Long) As Long

Declare Sub OutputDebugString Lib "kernel32" _
         Alias "OutputDebugStringA" _
         (ByVal lpOutputString As String)

Declare Sub DebugBreak Lib "kernel32" ()

Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Declare Function GetCurrentProcessId Lib "kernel32" () As Long

Declare Function GetLastError Lib "kernel32" () As Long

Declare Function SendMessage Lib "User32" _
         Alias "SendMessageA" _
         (ByVal hWnd As Long, _
          ByVal wMsg As Long, _
          ByVal wParam As Any, _
          lParam As Any) As Long

Declare Function PostMessage Lib "User32" _
         Alias "PostMessageA" _
         (ByVal hWnd As Long, _
         ByVal wMsg As Long, _
         ByVal wParam As Long, _
         ByVal lParam As Long) As Long

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                            Thread Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

' Note:  This declaration of CreateThread does not allow you to pass
'  thread attributes.  This is the CreateThread that should be used most
'  often because the parameter for the thread function is passed by
'  reference.
Public Declare Function CreateThread Lib "kernel32" _
         (ByVal lpThreadAttributes As Long, _
          ByVal dwStackSize As Long, _
          ByVal lpStartAddress As Any, _
          ByRef lpParameter As Any, _
          ByVal dwCreationFlags As Long, _
          ByRef lpThreadId As Long) As Long

' If you only are passing a single variable type like an integer or
'  string as the thread parameter, use this version of CreateThread.
' Note that I set the lpParameter type to long.
Public Declare Function CreateThread_ByValParam Lib "kernel32" _
         Alias "CreateThread" _
         (ByVal lpThreadAttributes As Long, _
          ByVal dwStackSize As Long, _
          ByVal lpStartAddress As Any, _
          ByVal lpParameter As Long, _
          ByVal dwCreationFlags As Long, _
          ByRef lpThreadId As Long) As Long

Public Declare Function TerminateThread Lib "kernel32" _
         (ByVal hThread As Long, _
          ByVal dwExitCode As Long) As Long

Declare Function SuspendThread Lib "kernel32" _
         (ByVal hThread As Long) As Long

Declare Function ResumeThread Lib "kernel32" _
         (ByVal hThread As Long) As Long

Declare Function GetExitCodeThread Lib "kernel32" _
         (ByVal hThread As Long, _
          ByRef lpExitCode As Long) As Long

Declare Sub ExitThread Lib "kernel32" (ByVal dwExitCode As Long)

Declare Function GetCurrentThread Lib "kernel32" () As Long

Declare Function GetCurrentThreadId Lib "kernel32" () As Long


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                            Event Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Declare Function CreateEvent Lib "kernel32" Alias "CreateEventA" _
         (ByVal lpEventAttributes As Long, _
          ByVal bManualReset As Long, _
          ByVal bInitialState As Long, _
          ByVal lpName As String) As Long

Declare Function OpenEvent Lib "kernel32" Alias "OpenEventA" _
         (ByVal dwDesiredAccess As Long, _
          ByVal bInheritHandle As Long, _
          ByVal lpName As String) As Long

Declare Function SetEvent Lib "kernel32" _
         (ByVal hEvent As Long) As Long

Declare Function ResetEvent Lib "kernel32" _
         (ByVal hEvent As Long) As Long

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                       Critical Section Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Declare Sub InitializeCriticalSection Lib "kernel32" _
         (ByRef lpCriticalSection As CRITICAL_SECTION)

Declare Sub DeleteCriticalSection Lib "kernel32" _
         (ByRef lpCriticalSection As CRITICAL_SECTION)

Declare Sub EnterCriticalSection Lib "kernel32" _
         (ByRef lpCriticalSection As CRITICAL_SECTION)

Declare Sub LeaveCriticalSection Lib "kernel32" _
         (ByRef lpCriticalSection As CRITICAL_SECTION)

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                       Synchronization Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Declare Function WaitForMultipleObjects Lib "kernel32" _
         (ByVal nCount As Long, _
          ByRef lpHandles As Long, _
          ByVal bWaitAll As Long, _
          ByVal dwMilliseconds As Long) As Long

Declare Function WaitForSingleObject Lib "kernel32" _
         (ByVal hHandle As Long, _
          ByVal dwMilliseconds As Long) As Long

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'              Process Creation and Manipulation Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Declare Function CreateProcess Lib "kernel32" Alias "CreateProcessA" _
         (ByVal lpApplicationName As String, _
          ByVal lpCommandLine As String, _
          ByVal lpProcessAttributes As Long, _
          ByVal lpThreadAttributes As Long, _
          ByVal bInheritHandles As Long, _
          ByVal dwCreationFlags As Long, _
          ByVal lpEnvironment As String, _
          ByVal lpCurrentDriectory As String, _
          ByRef lpStartupInfo As STARTUPINFO, _
          ByRef lpProcessInformation As PROCESS_INFORMATION) As Long

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                             Misc. Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Declare Function ReadProcessMemory Lib "kernel32" _
         (ByVal hProcess As Long, _
          ByVal lpBaseAddress As Long, _
          ByVal lpBuffer As String, _
          ByVal nSize As Long, _
          ByVal lpNumberOfBytesWritten As Long) As Long

Declare Function SetWindowText Lib "User32" Alias "SetWindowTextA" _
         (ByVal hWnd As Long, _
          ByVal lpString As String) As Long
